\8ACROSS.XPL	20-May-2017
\8-Across Solitaire Game
\Inspired by Margret Rosenberg, written by Loren Blaney.
\
\REVISIONS:
\JUN-27-99, Minor changes for XPL 2.4: Added Clear to prevent flashing card
\ images, winner's screen goes all the way to bottom.
\JUL-01-99, Swap Line and Text lines so that Nvidia dipslay works.
\ 1-Feb-2014: Modified for Raspberry Pi.
\03-Jul-2015: Modified for new GetMouse intrinsic. Speed up cards.
\20-May-2017: Remove "include codesr" to remove MoveMouse conflict.

\This uses 800x600 graphics, which is mode $103 on a VESA-compatible display.
\Since BIOS doesn't provide a mouse pointer for this mode, this code provides it

def	IntSize = 4;		\number of bytes in an integer

\Dimensions of images in 8across.bmp:
def	CardWidth = 71,		\playing card dimensions (pixels)
	CardHeight = 96,
	CardSpacing = 9,	\space between cards (pixels)
	HomeSpacing = 16,	\vertical space between cards in home position
	EraserHeight = CardHeight + HomeSpacing,
	EraserWidth = CardWidth + CardSpacing,
	ButtonWidth = 63,	\push button dimensions
	ButtonHeight = 23,
	ButtonSpacing = 30;	\center-to-center spacing, not space between

\Locations: pixel coordinates of upper-left corner
def	PileX = 130,	PileY = 30,	\the eight columns of cards go here
	HomeX = 30,	HomeY = 30,	\cards removed from pile stack up here
	UndoX = 34,	UndoY = 500,	\soft buttons: Undo, Deal and Exit
	MsgX = 2,	MsgY = 0,	\messages (text cursor coordinates)
	SlopXMax = CardSpacing-2,	\sloppiness with which cards are stacked
	SlopYMax = 4;			\set to zero if you're a neatness freak

int	ButtonImage(6),	\array of button images (2 per button, up and down)
	CardImage(54),	\array of card images (including blank and back side)
	EraserImage,	\blank card area, used as an eraser
	Home(4),	\cards moved to home position (removed from pile)
	CardOverlap(8),	\number of vertical pixels cards in each pile overlap
	Pile(8,52),	\array that holds cards in the pile formation
			\ 1..52 = card; 0 = no card; -1..-52 = face down card
	Remaining,	\number of cards remaining in the pile
	SaveRemaining,	\items saved for Undo command
	SaveHome(4),	\ used to restore values in corresponding variables
	SavePile(8),	\ (i.e: SavePile saves previous Pile data)
	Timer,		\elapsed time in seconds since beginning of game
	Timer0;		\determines when a full second has elapsed

def	Esc = $1B;	\escape control character code



proc	Exit(Msg);		\Display (error) message and terminate program
char	Msg;
begin
SetVid($03);			\restore normal text mode
Text(0, Msg);
CrLf(0);
exit;
end;	\Exit



func	GetSec;			\Get current time in seconds
return GetTime/1000000;



proc	ShowSprite(X0, Y0, Sp, Trans);	\Display a sprite
int	X0, Y0,	\coordinates of upper-left corner
	Sp,	\address of sprite data
	Trans;	\flag: make background (color 0) transparent instead of opaque
int	X, Y, C,
	W, H;	\width and height of sprite (pixels)
begin
W:= Sp(0);
H:= Sp(1);
Sp:= Sp + 2*IntSize;
for Y:= Y0 to Y0+H-1 do
    for X:= X0 to X0+W-1 do
	begin
	C:= Sp(0);			\get pixel's color
	Sp:= Sp + IntSize;
	if C ! ~Trans then Point(X, Y, C);
	end;
end;	\ShowSprite



proc	GetSprite(X0, Y0, W, H, Sp);	\Load sprite data array from screen
int	X0, Y0,	\upper-left corner coordinates of sprite on screen
	W, H,	\width and height of sprite (pixels)
	Sp;	\address of array in which to load sprite data
int	X, Y;
begin
Sp(0):= W;
Sp(1):= H;
Sp:= Sp + 2*IntSize;
for Y:= Y0 to Y0+H-1 do
    for X:= X0 to X0+W-1 do
	begin
	Sp(0):= ReadPix(X, Y);
	Sp:= Sp + IntSize;
	end;
end;	\GetSprite

\=============================== MOUSE ROUTINES ================================

int	MouseX, MouseY,		\current mouse coordinates (pixels)
	MouseX0, MouseY0,	\previous mouse coordinates (for backgnd image)
	MouseBut;		\mouse buttons: bit 0 set = left down, 1 = right
def	MouseW=12, MouseH=21;	\mouse pointer width and height (pixels)
int	MouseBg(2+MouseW*MouseH); \sprite: mouse pointer's background image



proc	DrawPointer(X, Y);	\Display mouse pointer at screen coordinates X,Y
int	X, Y;			\coordinates of upper-left corner of arrow
char	Ptr;
int	I, J;
begin
Ptr:=  ["#           ",
	"##          ",
	"#.#         ",
	"#..#        ",
	"#...#       ",
	"#....#      ",
	"#.....#     ",
	"#......#    ",
	"#.......#   ",
	"#........#  ",
	"#.........# ",
	"#......#####",
	"#...#..#    ",
	"#..##..#    ",
	"#.#  #..#   ",
	"##   #..#   ",
	"#     #..#  ",
	"      #..#  ",
	"       #..# ",
	"       #..# ",
	"        ##  "];

for J:= 0 to MouseH-1 do		\for all the rows...
    for I:= 0 to MouseW-1 do		\  for all the pixels in the row...
	case Ptr(J,I) & $7F of
	  ^#:	Point(I+X, J+Y, $3\Black\);
	  ^.:	Point(I+X, J+Y, $F\BWhite\)
	other	[];
end;	\DrawPointer



proc	ShowMouse(On);		\Turn mouse pointer on or off
int	On;	\flag: true = pointer on; false = pointer off
begin
if On then
	begin
	MouseX0:= MouseX;  MouseY0:= MouseY;	\record background's coordinates
	GetSprite(MouseX0, MouseY0, MouseW, MouseH, MouseBg); \save background
	DrawPointer(MouseX, MouseY);		\show mouse pointer
	end
else	begin
	ShowSprite(MouseX0, MouseY0, MouseBg, false);	\restore backgnd image
	end;
end;	\ShowMouse



proc	MoveMouse;		\Move mouse pointer
int	Mouse;
begin
Mouse:= GetMouse;
MouseX:= Mouse(0);  MouseY:= Mouse(1);  MouseBut:= Mouse(2);
if MouseX#MouseX0 ! MouseY#MouseY0 then			\mouse moved
	begin
	ShowSprite(MouseX0, MouseY0, MouseBg, false);	\restore backgnd image
	MouseX0:= MouseX;  MouseY0:= MouseY;		\new backgnd location
	GetSprite(MouseX0, MouseY0, MouseW, MouseH, MouseBg); \save new backgnd
	DrawPointer(MouseX, MouseY);			\show pointer at new loc
	end;
end;	\MoveMouse

\===============================================================================

proc	ShowTimer;		\Display the Timer as MM:SS
int	Min, Sec;
begin
ShowMouse(false);
Cursor(800/8-7, 0);
Min:= Timer/60;
Sec:= rem(0);
IntOut(6, Min);
ChOut(6, ^:);
if Sec < 10 then ChOut(6, ^0);
IntOut(6, Sec);
ShowMouse(true);
end;	\ShowTimer



proc	RunTimer;		\Update timer display once per second
begin
if Timer>=0 & Timer0#GetSec then
	begin
	Timer0:= GetSec;
	Timer:= Timer+1;
	ShowTimer;
	end;
end;	\RunTimer



proc	ClearColumn(Col);	\Erase a column of cards in the pile
int	Col;
int	X, Y;
begin
X:= PileX + Col*(CardWidth+CardSpacing);
Y:= PileY;
repeat	ShowSprite(X, Y, EraserImage, false);
	Y:= Y + EraserHeight;
until	Y + EraserHeight >= 600;
ShowSprite(X, 600-EraserHeight, EraserImage, false);
end;	\ClearColumn



proc	ShowCard(N, X, Y);	\Display card N at location X,Y
int	N,	\card number (1-52), 0 is blank, negative numbers are face down
	X, Y;	\screen coordinates (pixels)
int	SlopX, SlopY;
begin
SlopX:= Ran(SlopXMax+1);
SlopY:= Ran(SlopYMax+1);
ShowSprite(X+SlopX, Y+SlopY, CardImage(if N<0 then 53 \backside\ else N), true);
end;	\ShowCard;



proc	ShowCards(Locs);	\Display cards in pile and home position
int	Locs;	\bit position indicates columns and home positions to update
int	H, C, X, Y, T, B, X1, Y1, N;
begin
\Show home positions
for H:= 0 to 4-1 do
    if 1<<(H+8) & Locs then
	begin
	Y:= HomeY + H*(CardHeight+HomeSpacing);

	\Clear area around home pocket
	ShowSprite(HomeX, Y, EraserImage, false);

	\Show pocket
	ShowSprite(HomeX+SlopXMax/2, Y+SlopYMax/2, CardImage(0), false);

	\Show all cards up to the top card
	T:= Home(H);		\get top card
	B:= Rem((T-1)/13);	\get the rank 0..12
	B:= T - B;		\the bottom card (the ace in the correct suit)
	for C:= B to T do ShowCard(C, HomeX, Y);
	end;

\Reduce the amount of overlap in columns that have lots of cards
for X:= 0 to 8-1 do
    begin
    if 1<<X & Locs then
	begin
	N:= 0;			\count the number of cards in this column (N)
	for Y:= 0 to 52-1 do
		begin
		C:= Pile(X,Y);		\get card
		if C # 0 then N:= N+1 else Y:= 52;
		end;
	CardOverlap(X):= if N <= 15 then 30
			 else (600-PileY-CardHeight-10) / (N-1);
	end;
    end;

\Show all the cards in the pile
for X:= 0 to 8-1 do
    if 1<<X & Locs then
	begin
	ClearColumn(X);
	X1:= PileX + X*(CardWidth+CardSpacing);
	ShowSprite(X1+SlopXMax/2, PileY+SlopYMax/2, CardImage(0), false);
	for Y:= 0 to 52-1 do
		begin
		C:= Pile(X,Y);		\get card
		if C # 0 then
			begin
			Y1:= PileY + Y*CardOverlap(X);
			ShowCard(C, X1, Y1);
			DelayUS(16667);	\go slow for effect
			end
		else Y:= 52;		\quit 'for' loop
		end;
	end;
end;	\ShowCards

\===============================================================================

proc	Exch(A, B);	\Exchange the contents of locations A and B
int	A, B;
int	T;
begin
T:= A(0);
A(0):= B(0);
B(0):= T;
end;	\Exch



proc	Undo;		\Undo last move (toggles)
int	X, Y;
begin
Exch(@Remaining, @SaveRemaining);
for Y:= 0 to 4-1 do Exch(@Home(Y), @SaveHome(Y));
for X:= 0 to 8-1 do
    for Y:= 0 to 52-1 do
	Exch(@Pile(X,Y), @SavePile(X,Y));
ShowCards($FFF);
end;	\Undo



proc	DoDeal;		\Restart program if Deal button was pressed
Restart;



proc	ShowButton(N, Down);	\Show button N in either its up or down position
int	N, Down;
\ N  Button
\ 0  Undo
\ 1  Deal
\ 2  Exit
begin
ShowMouse(false);
ShowSprite(UndoX, UndoY+ButtonSpacing*N,
	ButtonImage(N*2+(if Down then 1 else 0)), false);
ShowMouse(true);
end;	\ShowButton



proc	HandleKeyCommands;	\Check for and handle shortcut keystrokes
int	Ch;
begin
if KeyHit then
	begin
	Ch:= GetKey;
	ShowMouse(false);	\(necessary for keyboard "U" command)
	case Ch of
	  ^U,^u,-22:	Undo;		\(Alt+U = -22)
	  ^D,^d,-32:	DoDeal;		\(Alt+D = -32)
	  ^X,^x,-45,Esc:Exit(" ")	\(Alt+X = -45)
	other	  	[];
	ShowMouse(true);
	end;
end;	\HandleKeyCommands



proc	GetMouseClick;		\Wait for the mouse to click on something
int	N, N0, Pressed;
begin
ShowMouse(true);
loop begin
repeat	MoveMouse;
	HandleKeyCommands;
	RunTimer;
until	MouseBut & $03;		\a mouse button is pressed

\Get soft button number (N0) (if any) that mouse pointer is on
\If mouse was on button N then record button number in N0
N0:= 0;
for N:= 0 to 3-1 do
    if MouseX>=UndoX & MouseX<UndoX+ButtonWidth &
       MouseY>=UndoY+N*ButtonSpacing &
       MouseY<UndoY+N*ButtonSpacing+ButtonHeight then N0:= N;

Pressed:= false;
repeat	MoveMouse;
	HandleKeyCommands;
	RunTimer;
	\If mouse pointer is on original soft button (N0) then show it pressed
	if MouseX>=UndoX & MouseX<UndoX+ButtonWidth &
	   MouseY>=UndoY+N0*ButtonSpacing &
	   MouseY< UndoY+N0*ButtonSpacing+ButtonHeight then
		begin
		if not Pressed then		\only show button if it changes
			begin
			ShowButton(N0, true);	\show pressed button
			Pressed:= true;		\indicate it has been displayed
			end;
		end
	else	begin	\mouse pointer is not on original button: show button up
		if Pressed then
			begin
			ShowButton(N0, false);	\show unpressed button
			Pressed:= false;
			end;
		end;
until	(MouseBut&$03) = 0;			\both buttons are released

\If a soft button is down then execute its function
if Pressed then
	begin
	ShowButton(N0, false);		\display unpressed button
	ShowMouse(false);		\(necessary for Undo command)
	case N0 of
	  0:	Undo;
	  1:	DoDeal;
	  2:	Exit(" ")
	other	[];
	ShowMouse(true);
	end
else quit;
end;	\loop

ShowMouse(false);
end;	\GetMouseClick



proc	Wait(T);	\Wait T 60ths of a second or until a mouse click
int	T;
begin
ShowMouse(true);
loop	begin
	MoveMouse;
	HandleKeyCommands;
	RunTimer;
	DelayUS(16667);
	T:= T-1;
	if T <= 0 then quit;
	if MouseBut & $03 then quit;
	end;
ShowMouse(false);
end;	\Wait



proc	ShowWin;	\Show "1000 card pickup" for winning screen
int	Mouse, N, X, Y, T;
begin
T:= 1000;
loop	begin
	Mouse:= GetMouse;
	MouseBut:= Mouse(2);
	ShowMouse(true);
	HandleKeyCommands;
	ShowMouse(false);
	if T < 940 then		\beware of double clicking
		if MouseBut & $03 then quit;				\-->
	DelayUS(8333);
	T:= T-1;
	if T <= 0 then quit;						\-->
	N:= Ran(52)+1;		\1..52
	if Ran(5) = 0 then N:= -N;
	X:= Ran(800-CardWidth);
	Y:= Ran(600-CardHeight-PileY) + PileY;
	ShowSprite(X, Y, CardImage(if N<0 then 53 \backside\ else N), true);
	end;
end;	\ShowWin

\-------------------------------------------------------------------------------

proc	ShowMsg(Str);	\Display an error message
char	Str;
begin
Cursor(MsgX, MsgY);
Text(6, Str);		\(device 0 doesn't always work with VESA graphics)
end;	\ShowMsg



proc	Beep;		\A not-too-obnoxious beep
begin
\\Sound(false, 1, 1000);	\synchronize with system timer to make tone a
\\Sound(true, 1, 3000);	\ consistent duration and a consistent sound.
end;	\Beep



proc	ErrorMsg(Str);	\Display an error message
char	Str;
begin
Beep;
ShowMsg(Str);
end;	\ErrorMsg



proc	SaveState;	\Save current state for Undo command
int	X, Y;
begin
SaveRemaining:= Remaining;
for Y:= 0 to 4-1 do SaveHome(Y):= Home(Y);
for X:= 0 to 8-1 do
    for Y:= 0 to 52-1 do
	SavePile(X,Y):= Pile(X,Y);
end;	\SaveState

\-------------------------------------------------------------------------------

proc	MakeMove;	\Make a (legal) move
int	X0, Y0,		\pile's column and row coordinates (top card to move)
	X1, Y1,		\column and row coordinates (-1) of destination
	C0, C1,		\card values at source (C0) and destination (C1)
	N;		\scratch


	proc	MoveStack;	\Move stack of cards from X0,Y0 to X1,Y1
	begin
	N:= 0;
	while Pile(X0,Y0+N) # 0 do		\last card has been moved
		begin
		Pile(X1,Y1+N):= Pile(X0,Y0+N);	\move card to new location
		Pile(X0,Y0+N):= 0;		\remove card from old location
		N:= N+1;			\next card
		end;
	end;	\MoveStack


begin	\MakeMove
\Ignore invalid moves and display feedback messages indicating reason for error
loop	begin
	loop	begin				\click on card(s) to move
		ShowMsg("Click on a card to move     ");
		GetMouseClick;
		\Ignore click if it's not on the pile
		if MouseX<PileX ! MouseX>=PileX+8*(CardWidth+CardSpacing) !
		   MouseY<PileY then
			ErrorMsg("Select a card from the pile ")
		else	begin
			\Convert pixel coordinates to pile's column and row
			X0:= (MouseX - PileX) / (CardWidth + CardSpacing);
			Y0:= (MouseY - PileY) / CardOverlap(X0);

			\If clicking on bottom part of bottom card then move up
			for N:= 0 to CardHeight/CardOverlap(X0) do
				if Pile(X0,Y0)=0 & Y0>0 then Y0:= Y0-1;

			\Ignore click if it's not on a card
			C0:= Pile(X0,Y0);		\get card's value
			if C0 = 0 then
				ErrorMsg("Select a card               ")

			\Ignore click if the card is not facing up
			else if C0 < 0 then
				ErrorMsg("Only face-up cards can move ")

			\Otherwise exit loop with valid card selected from pile
			else quit;					\-->
			end;

		Wait(120);			\provide time to read message
		end;	\loop - repeat until valid card is selected

	\Provide feedback to player
	ShowMsg("Click on a column to move to");

	\Click on target (move-to) location
	GetMouseClick;

	\If target location is in the pile then...
	if MouseX>=PileX & MouseX<PileX+8*(CardWidth+CardSpacing) &
	   MouseY>=PileY then
		begin
		\Get the card at the bottom of the stack
		X1:= (MouseX - PileX) / (CardWidth + CardSpacing);

		\Move Y1 up to first card at bottom of stack or to top of column
		Y1:= 52-1;
		while Pile(X1,Y1)=0 & Y1>0 do Y1:= Y1-1;
		C1:= Pile(X1,Y1);	\get card value

		\If target column contains a card then...
		if C1 # 0 then
			begin
			\If target column = source column then...
			if X1 = X0 then
			    begin
			    \If source card (C0) is the bottom card then...
			    if Pile(X0,Y0+1) = 0 then
				begin
				\If there's a legal move to the home posn then
				for Y1:= 0 to 4-1 do
				\(Don't put ace of spades on king of clubs, etc)
					begin
					if C0=Home(Y1)+1 & Rem(C0/13)#1\ace\ !
					   Rem(C0/13)=1\ace\ & Home(Y1)=0 then
						begin	\move card to home posn
						SaveState;
						Home(Y1):= C0;	\move card
						Pile(X0,Y0):= 0;
						Remaining:= Remaining-1;
						X1:= Y1+8;	\for ShowCards
						quit;			\-->
						end;
					end;
				end;
			    ErrorMsg("Can't move to same column   ");
			    end
			\Else if source card = target card +/-1 then...
			else if C0=C1-1 & Rem(C1/13)#1 \not an ace\  !
				C0=C1+1 & Rem(C1/13)#0 \not a king\ then
				begin
				SaveState;
				Y1:= Y1+1;	\move to below target card
				MoveStack;
				quit;					\-->
				end
			else	ErrorMsg("Cards must be in sequence   ");
			end
		else	begin			\target column is blank
			\If source card is not a king then error
			if Rem(C0/13) # 0 then
				ErrorMsg("Only a king can move here   ")

			\Else if king is not off bottom or top then error
			else if Pile(X0,Y0+1)#0 & Y0#0 then
				ErrorMsg("King must be at the bottom  ")

			\Else move card(s) and quit loop
			else	begin
				SaveState;
				MoveStack;
				quit;					\-->
				end;
			end;
		end

	\Else if target location is in the home area then...
	else if MouseX>=HomeX & MouseX<HomeX+4*(CardWidth+CardSpacing) &
		MouseY>=HomeY & MouseY<HomeY+4*(CardHeight+HomeSpacing) then
		begin
		Y1:= (MouseY - HomeY) / (CardHeight + HomeSpacing);
		C1:= Home(Y1);				\get card's value

		\If source card is not from the bottom of the stack then error
		if Pile(X0,Y0+1) # 0 then
			ErrorMsg("Card must be at the bottom  ")

		\Else if target location contains card & source card does not
		\ equal target card+1 (or top already has a king) then error
		else if C1#0 & (C0#C1+1 ! Rem(C1/13)=0\king\) then
			ErrorMsg("Cards must be in sequence   ")

		\Else if target location is blank & source card is not an ace
		\ then error
		else if C1=0 & Rem(C0/13)#1\ace\ then
			ErrorMsg("Only an ace can move here   ")

		\Else move card to home position
		else	begin
			SaveState;
			Home(Y1):= C0;		\move card
			Pile(X0,Y0):= 0;
			Remaining:= Remaining-1;
			X1:= Y1 + 8;		\for ShowCards
			quit;						\-->
			end;
		end

	\Else target location is illegal
	else ErrorMsg("Move to column or home stack");

	Wait(120);
	end;	\loop - repeat until valid move is made

ShowMsg("                            ");	\erase prompt message for safety

\If there is a face-down card at the bottom of the stack, turn it up
if Y0 > 0 then
	if Pile(X0,Y0-1) < 0 then Pile(X0,Y0-1):= -Pile(X0,Y0-1);

ShowCards(1<<X0 ! 1<<X1);			\show result
end;	\MakeMove

\-------------------------------------------------------------------------------

proc	Deal;		\Put all 52 cards into the initial pile configuration
\			  and empty the Home array
\
\     0  1  2  3  4  5  6  7  X
\ 0   U  D  U  D  U  D  U  D		U = Face up
\ 1   D  U  D  U  D  U  D  U		D = Face down
\ 2   U  D  U  D  U  D  U  D
\ 3      U  D  U  D  U  D  U
\ 4         U  D  U  D  U  D
\ 5            U  D  U  D  U
\ 6               U  D  U  D
\ 7                  U  D  U
\ 8                     U  D
\ 9                        U
\ Y
\
int	X, Y;		\coordinates in the pile
char	Deck(52);	\array of card values 0..51
int	Cnt, T;


	func	GetCard;	\Randomly select a card (1-52)
	int	Loc, Card, I;
	begin
	Loc:= Ran(Cnt);			\randomly select a location in the deck
	Card:= Deck(Loc);		\get card value at this location
	for I:= Loc to Cnt-2 do		\remove selected card by moving all the
		Deck(I):= Deck(I+1);	\ rest of the cards down
	Cnt:= Cnt-1;			\one less card location to select from
	return Card;			\return card value (1..52)
	end;	\GetCard


begin	\Deal
Cnt:= 52;			\Set up deck with all 52 card values (1..52)
for X:= 0 to 52-1 do Deck(X):= X+1;

for Y:= 0 to 52-1 do		\initialize Pile with "no cards"
    for X:= 0 to 8-1 do
	Pile(X,Y):= 0;

for Y:= 0 to 9 do		\set up Pile with starting configuration
    for X:= 0 to 8-1 do		\ (shown above)
	begin
	CardOverlap(X):= 30;
	if Y <= X+2 then
		begin
		Pile(X,Y):= if (X+Y) & 1 then -GetCard else GetCard;
		for T:= 0 to 3-1 do DelayUS(16667);	\go slow for effect
		ShowCard(Pile(X,Y), PileX+X*(CardWidth+CardSpacing),
			PileY+Y*CardOverlap(X));
		end;
	end;

for Y:= 0 to 4-1 do Home(Y):= 0; \no cards in Home positions
end;	\Deal

\-------------------------------------------------------------------------------

proc	PlayGame;		\Play one game of Solitaire
int	N, H;
begin
ShowMouse(false);		\turn off mouse pointer while writing to screen
Clear;
Move(0, 18);   Line(800-1, 18, 3\Black\);
Cursor(40, 0);
Text(6, "8-Across Solitaire");

for N:= 0 to 8-1 do		\show empty card pockets
	ShowSprite(PileX+N*(CardWidth+CardSpacing)+SlopXMax/2, PileY+SlopYMax/2,
		   CardImage(0), false);
for H:= 0 to 4-1 do
       ShowSprite(HomeX+SlopXMax/2, HomeY+H*(CardHeight+HomeSpacing)+SlopYMax/2,
		CardImage(0), false);
for N:= 0 to 3-1 do			\show buttons
	ShowSprite(UndoX, UndoY+ButtonSpacing*N, ButtonImage(N*2), false);

Deal;

Remaining:= 52;			\start with a full deck in the pile
SaveState;			\make sure Undo command doesn't restore garbage

Timer0:= GetSec;		\wait for the beginning of a one-second period
repeat until Timer0 # GetSec;
Timer0:= GetSec;
Timer:= 0;			\start the Timer
ShowMouse(true);
ShowTimer;			\show "00:00"
ShowMouse(false);

repeat MakeMove until Remaining <= 0;
ShowMsg("Congratulations!            ");
ShowWin;
Timer:= -1;			\stop timer
GetMouseClick;
end;	\PlayGame

\-------------------------------------------------------------------------------

proc	OpenInFile(FN);		\Open specified file for input
char	FN;	\file name string
int	H;	\handle
begin
H:= FOpen(FN, 0);
FSet(H, ^I);
OpenI(3);
end;	\OpenInFile



proc	LoadBmp;		\Load opened .bmp file into display memory
int	Width, Height,	\dimensions of .bmp image in pixels
	R, G, B,	\colors
	X, Y,		\screen coordinates (pixels)
	I, T;		\index and scratch
begin
for Y:= 0 to 17 do X:= ChIn(3);	\skip unused header info
Width:= ChIn(3) + ChIn(3)<<8;	\0..32764 (ample range)
for Y:= 0 to 2-1 do X:= ChIn(3);\skip
Height:= ChIn(3) + ChIn(3)<<8;	\0..32767 (ample range)
for Y:= 24 to 53 do X:= ChIn(3);\skip

for I:= 0 to 16-1 do
	[B:= ChIn(3);
	 G:= ChIn(3);
	 R:= ChIn(3);
	 T:= ChIn(3);
	 SetPalette(I, R, G, B);
	];

for Y:= Height-1 downto 0 do	\.bmp files are upside down!
    for X:= 0 to Width-1 do
	begin
	T:= ChIn(3);
	Point(X, Y, T>>4);
	X:= X+1;
	Point(X, Y, T&$F);
	end;
end;	\LoadBmp



proc	Initialize;
int	I, X, Y, X0, Y0;
begin
TrapC(true);		\force user to make a clean exit, restoring video mode
SetVid($103);				\set graphics
if not OpenMouse then Exit("This program requires a mouse.");
string 0;
OpenInFile("8across.bmp");		\load card images into display memory
LoadBmp;

\Copy card images into regular memory
I:= 0;
for Y:= 0 to 6-1 do
    for X:= 0 to 9-1 do
      begin				\CardImage(0) = Empty card pocket
      GetSprite(X*CardWidth, Y*CardHeight, CardWidth, CardHeight, CardImage(I));
      I:= I+1;
      end;

\Copy a blank image area to use as an eraser
GetSprite(9*CardWidth, 100, EraserWidth, EraserHeight, EraserImage);

\Copy button images to memory
X0:= 639;   Y0:= 28;			\upper-left corner of button images
I:= 0;
for Y:= 0 to 3-1 do
    for X:= 0 to 2-1 do
	begin
	GetSprite(X0+X*ButtonWidth, Y0+Y*ButtonHeight, ButtonWidth,
		ButtonHeight, ButtonImage(I));
	I:= I+1;
	end;
Clear;
end;	\Initialize

\===============================================================================

int	I;
begin	\Main
for I:= 0 to 54-1 do		\CardImage(53) = Card backside
	CardImage(I):= Reserve((CardWidth*CardHeight+2)*IntSize);
for I:= 0 to 6-1 do
	ButtonImage(I):= Reserve((ButtonWidth*ButtonHeight+2)*IntSize);
EraserImage:= Reserve((EraserWidth*EraserHeight+2)*IntSize);
for I:= 0 to 8-1 do
	SavePile(I):= Reserve(52*IntSize);

if not Rerun then Initialize;		\"Deal" button restarts program
MouseX:= 0;  MouseY:= 0;		\initialize mouse position
ShowMouse(true);			\display mouse pointer
loop PlayGame;
end;	\Main
